@@ -27,12 +27,12 @@ except ImportError:
 
 import keystone.api
 from keystone import exception
+from keystone.oauth2 import handlers as oauth2_handlers
+from keystone.receipt import handlers as receipt_handlers
 from keystone.server.flask import common as ks_flask
 from keystone.server.flask.request_processing import json_body
 from keystone.server.flask.request_processing import req_logging
 
-from keystone.receipt import handlers as receipt_handlers
-
 LOG = log.getLogger(__name__)
 
 
@@ -75,16 +75,16 @@ def _handle_keystone_exception(error):
     # TODO(adriant): register this with its own specific handler:
     if isinstance(error, exception.InsufficientAuthMethods):
         return receipt_handlers.build_receipt(error)
+    elif isinstance(error, exception.OAuth2Error):
+        return oauth2_handlers.build_response(error)
 
     # Handle logging
     if isinstance(error, exception.Unauthorized):
         LOG.warning(
             "Authorization failed. %(exception)s from %(remote_addr)s",
             {'exception': error, 'remote_addr': flask.request.remote_addr})
-    elif isinstance(error, exception.UnexpectedError):
-        LOG.exception(str(error))
     else:
-        LOG.warning(str(error))
+        LOG.exception(str(error))
 
     # Render the exception to something user "friendly"
     error_message = error.args[0]
